@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,20 +21,39 @@
 
 import hashlib
 import sys
+from abc import ABCMeta, abstractmethod
 
 from nuitka import Options
 from nuitka.__past__ import iterItems
-from nuitka.Constants import constant_builtin_types
+from nuitka.Builtins import (
+    builtin_anon_codes,
+    builtin_anon_values,
+    builtin_exception_values_list
+)
 from nuitka.PythonVersions import python_version
+from nuitka.utils.InstanceCounters import counted_del, counted_init
 
 from .Namify import namifyConstant
 
 
-# Many methods won't use self, but it's the interface. pylint: disable=R0201
+class ContextMetaClass(ABCMeta):
+    pass
 
-class TempMixin:
+
+# For Python2/3 compatible source, we create a base class that has the metaclass
+# used and doesn't require making a choice.
+ContextMetaClassBase = ContextMetaClass(
+    "ContextMetaClassBase",
+    (object,),
+    {}
+)
+
+
+# Many methods won't use self, but it's the interface. pylint: disable=no-self-use
+
+class TempMixin(object):
     # Lots of details, everything gets to store bits here, to indicate
-    # code generation states, and there are many, pylint: disable=R0902
+    # code generation states, and there are many, pylint: disable=too-many-instance-attributes
 
     def __init__(self):
         self.tmp_names = {}
@@ -58,6 +77,8 @@ class TempMixin:
 
         self.preserver_variable_counts = set()
 
+        self.cleanup_names = []
+
     def formatTempName(self, base_name, number):
         if number is None:
             return "tmp_{name}".format(
@@ -90,6 +111,12 @@ class TempMixin:
             number    = number
         )
 
+    def getIntResName(self):
+        return self.allocateTempName("res", "int", unique = True)
+
+    def getBoolResName(self):
+        return self.allocateTempName("result", "bool", unique = True)
+
     def hasTempName(self, base_name):
         return base_name in self.tmp_names
 
@@ -164,6 +191,9 @@ class TempMixin:
             number = result
         )
 
+    def getLabelCount(self, label):
+        return self.labels.get(label, 0)
+
     def needsExceptionVariables(self):
         return self.needs_exception_variables
 
@@ -210,8 +240,30 @@ class TempMixin:
     def setFalseBranchTarget(self, label):
         self.false_target = label
 
+    def getCleanupTempnames(self):
+        return self.cleanup_names[-1]
 
-class CodeObjectsMixin:
+    def addCleanupTempName(self, tmp_name):
+        assert tmp_name not in self.cleanup_names[-1], tmp_name
+
+        self.cleanup_names[-1].append(tmp_name)
+
+    def removeCleanupTempName(self, tmp_name):
+        assert tmp_name in self.cleanup_names[-1], tmp_name
+        self.cleanup_names[-1].remove(tmp_name)
+
+    def needsCleanup(self, tmp_name):
+        return tmp_name in self.cleanup_names[-1]
+
+    def pushCleanupScope(self):
+        self.cleanup_names.append([])
+
+    def popCleanupScope(self):
+        assert not self.cleanup_names[-1]
+        del self.cleanup_names[-1]
+
+
+class CodeObjectsMixin(object):
     def __init__(self):
         # Code objects needed made unique by a key.
         self.code_objects = {}
@@ -219,24 +271,21 @@ class CodeObjectsMixin:
     def getCodeObjects(self):
         return sorted(iterItems(self.code_objects))
 
-    def getCodeObjectHandle(self, code_object, filename, line_number,
-                            is_optimized, new_locals, has_closure,
-                            future_flags):
-
+    def getCodeObjectHandle(self, code_object):
         key = (
-            filename,
+            code_object.getFilename(),
             code_object.getCodeObjectName(),
-            line_number,
+            code_object.getLineNumber(),
             code_object.getVarNames(),
             code_object.getArgumentCount(),
             code_object.getKwOnlyParameterCount(),
-            code_object.getKind(),
-            is_optimized,
-            new_locals,
+            code_object.getCodeObjectKind(),
+            code_object.getFlagIsOptimizedValue(),
+            code_object.getFlagNewLocalsValue(),
             code_object.hasStarListArg(),
             code_object.hasStarDictArg(),
-            has_closure,
-            future_flags
+            code_object.getFlagHasClosureValue(),
+            code_object.getFutureSpec().asFlags()
         )
 
         if key not in self.code_objects:
@@ -247,28 +296,221 @@ class CodeObjectsMixin:
     if python_version < 300:
         def _calcHash(self, key):
             hash_value = hashlib.md5(
-                "%s%s%d%s%d%d%s%s%s%s%s%s%s" % key
+                "%s-%s-%d-%s-%d-%d-%s-%s-%s-%s-%s-%s-%s" % key
             )
 
             return hash_value.hexdigest()
     else:
         def _calcHash(self, key):
             hash_value = hashlib.md5(
-                ("%s%s%d%s%d%d%s%s%s%s%s%s%s" % key).encode("utf-8")
+                ("%s-%s-%d-%s-%d-%d-%s-%s-%s-%s-%s-%s-%s" % key).encode("utf-8")
             )
 
             return hash_value.hexdigest()
 
 
-class PythonContextBase:
+class PythonContextBase(ContextMetaClassBase):
+    @counted_init
     def __init__(self):
         self.source_ref = None
 
-    def isCompiledPythonModule(self):
-        return False
+        self.current_source_ref = None
+        self.last_source_ref = None
+
+    __del__ = counted_del()
+
+    def getCurrentSourceCodeReference(self):
+        return self.current_source_ref
+
+    def setCurrentSourceCodeReference(self, value):
+        result = self.current_source_ref
+        self.current_source_ref = value
+
+        if value is not None:
+            self.last_source_ref = result
+
+        return result
+
+    def getLastSourceCodeReference(self):
+        result = self.last_source_ref
+        # self.last_source_ref = None
+        return result
+
+    def isUsed(self, tmp_name):
+        if tmp_name.startswith("tmp_unused_"):
+            return False
+        else:
+            return True
+
+    @abstractmethod
+    def getConstantCode(self, constant):
+        pass
+
+    @abstractmethod
+    def getModuleCodeName(self):
+        pass
+
+    @abstractmethod
+    def getModuleName(self):
+        pass
+
+    @abstractmethod
+    def addHelperCode(self, key, code):
+        pass
+
+    @abstractmethod
+    def hasHelperCode(self, key):
+        pass
+
+    @abstractmethod
+    def addDeclaration(self, key, code):
+        pass
+
+    @abstractmethod
+    def pushFrameVariables(self, frame_variables):
+        pass
+
+    @abstractmethod
+    def popFrameVariables(self):
+        pass
+
+    @abstractmethod
+    def getFrameVariableTypeDescriptions(self):
+        pass
+
+    @abstractmethod
+    def getFrameVariableTypeDescription(self):
+        pass
+
+    @abstractmethod
+    def getFrameVariableTypeDescriptionName(self):
+        pass
+
+    @abstractmethod
+    def getFrameVariableCodeNames(self):
+        pass
+
+    @abstractmethod
+    def allocateTempName(self, base_name, type_name = "PyObject *",
+                         unique = False):
+        pass
+
+    @abstractmethod
+    def getIntResName(self):
+        pass
+
+    @abstractmethod
+    def getBoolResName(self):
+        pass
+
+    @abstractmethod
+    def hasTempName(self, base_name):
+        pass
+
+    @abstractmethod
+    def forgetTempName(self, tmp_name):
+        pass
+
+    @abstractmethod
+    def getTempNameInfos(self):
+        pass
+
+    @abstractmethod
+    def getExceptionEscape(self):
+        pass
+
+    @abstractmethod
+    def setExceptionEscape(self, label):
+        pass
+
+    @abstractmethod
+    def getLoopBreakTarget(self):
+        pass
+
+    @abstractmethod
+    def setLoopBreakTarget(self, label):
+        pass
+
+    @abstractmethod
+    def getLoopContinueTarget(self):
+        pass
+
+    @abstractmethod
+    def setLoopContinueTarget(self, label):
+        pass
+
+    @abstractmethod
+    def allocateLabel(self, label):
+        pass
+
+    @abstractmethod
+    def needsExceptionVariables(self):
+        pass
+
+    @abstractmethod
+    def markAsNeedsExceptionVariables(self):
+        pass
+
+    @abstractmethod
+    def allocateExceptionKeeperVariables(self):
+        pass
+
+    @abstractmethod
+    def getExceptionKeeperVariables(self):
+        pass
+
+    @abstractmethod
+    def setExceptionKeeperVariables(self, keeper_vars):
+        pass
+
+    @abstractmethod
+    def addExceptionPreserverVariables(self, count):
+        pass
+
+    @abstractmethod
+    def getTrueBranchTarget(self):
+        pass
+
+    @abstractmethod
+    def getFalseBranchTarget(self):
+        pass
+
+    @abstractmethod
+    def setTrueBranchTarget(self, label):
+        pass
+
+    @abstractmethod
+    def setFalseBranchTarget(self, label):
+        pass
+
+    @abstractmethod
+    def getCleanupTempnames(self):
+        pass
+
+    @abstractmethod
+    def addCleanupTempName(self, tmp_name):
+        pass
+
+    @abstractmethod
+    def removeCleanupTempName(self, tmp_name):
+        pass
+
+    @abstractmethod
+    def needsCleanup(self, tmp_name):
+        pass
+
+    @abstractmethod
+    def pushCleanupScope(self):
+        pass
+
+    @abstractmethod
+    def popCleanupScope(self):
+        pass
 
 
 class PythonChildContextBase(PythonContextBase):
+    # Base classes can be abstract, pylint: disable=abstract-method
+
     def __init__(self, parent):
         PythonContextBase.__init__(self)
 
@@ -292,9 +534,28 @@ class PythonChildContextBase(PythonContextBase):
     def addDeclaration(self, key, code):
         self.parent.addDeclaration(key, code)
 
+    def pushFrameVariables(self, frame_variables):
+        return self.parent.pushFrameVariables(frame_variables)
+
+    def popFrameVariables(self):
+        return self.parent.popFrameVariables()
+
+    def getFrameVariableTypeDescriptions(self):
+        return self.parent.getFrameVariableTypeDescriptions()
+
+    def getFrameVariableTypeDescription(self):
+        return self.parent.getFrameVariableTypeDescription()
+
+    def getFrameVariableTypeDescriptionName(self):
+        return self.parent.getFrameVariableTypeDescriptionName()
+
+    def getFrameVariableCodeNames(self):
+        return self.parent.getFrameVariableCodeNames()
 
 
 def _getConstantDefaultPopulation():
+    # Lots of cases, pylint: disable=too-many-branches
+
     # Note: Can't work with set here, because we need to put in some values that
     # cannot be hashed.
 
@@ -316,6 +577,7 @@ def _getConstantDefaultPopulation():
         "__module__",
         "__class__",
         "__name__",
+        "__package__",
         "__metaclass__",
         "__dict__",
         "__doc__",
@@ -335,8 +597,23 @@ def _getConstantDefaultPopulation():
         "compile",
         "range",
         "open",
+        "sum",
         "format",
         "__import__",
+        "bytearray",
+        "staticmethod",
+        "classmethod",
+
+        # Arguments of __import__ built-in used in helper code.
+        "name",
+        "globals",
+        "locals",
+        "fromlist",
+        "level",
+
+        # Meta path based loader.
+        "read",
+        "rb"
     ]
 
     if python_version >= 300:
@@ -353,7 +630,18 @@ def _getConstantDefaultPopulation():
             "file"
         )
 
-    if python_version >= 330:
+        # For Python3 "bytes" built-in.
+        result.append(
+            "bytes"
+        )
+
+        # For Python3 "__name__" to "__package__" parsing
+        result.append(
+            '.'
+        )
+
+
+    if python_version >= 300:
         # Modules have that attribute starting with 3.3
         result.append(
             "__loader__"
@@ -365,7 +653,7 @@ def _getConstantDefaultPopulation():
             "send"
         )
 
-    if python_version >= 330:
+    if python_version >= 300:
         result += (
             # YIELD_FROM uses this
             "throw",
@@ -427,6 +715,13 @@ def _getConstantDefaultPopulation():
             "ignore"
         )
 
+    if python_version >= 340:
+        # Setting the __spec__ module attribute.
+        result += [
+            "__spec__",
+            "_initializing"
+        ]
+
     if python_version >= 350:
         # Patching the types module.
         result.append(
@@ -438,10 +733,15 @@ def _getConstantDefaultPopulation():
             sys.executable
         )
 
+    if python_version >= 370:
+        result.append(
+            "__class_getitem__"
+        )
+
     return result
 
 
-class PythonGlobalContext:
+class PythonGlobalContext(object):
     def __init__(self):
         self.constants = {}
         self.constant_use_count = {}
@@ -456,8 +756,9 @@ class PythonGlobalContext:
         self.needs_exception_variables = False
 
     def getConstantCode(self, constant):
-        # Use in user code, or for constants building code itself
-
+        # Use in user code, or for constants building code itself, many
+        # constant types get special code immediately.
+        # pylint: disable=too-many-branches
         if constant is None:
             key = "Py_None"
         elif constant is True:
@@ -466,22 +767,44 @@ class PythonGlobalContext:
             key = "Py_False"
         elif constant is Ellipsis:
             key = "Py_Ellipsis"
-        elif constant in constant_builtin_types:
-            type_name = constant.__name__
+        elif constant is NotImplemented:
+            key = "Py_NotImplemented"
+        elif type(constant) is type:
+            # TODO: Maybe make this a mapping in nuitka.Builtins
 
-            if constant is int and python_version >= 300:
-                type_name = "long"
-
-            if constant is str and python_version < 300:
-                type_name = "string"
-
-            if constant is str and python_version > 300:
-                type_name = "unicode"
-
-            if type_name != "NoneType":
-                key = "(PyObject *)&Py%s_Type" % type_name.title()
-            else:
+            if constant is None:
                 key = "(PyObject *)Py_TYPE( Py_None )"
+            elif constant is object:
+                key = "(PyObject *)&PyBaseObject_Type"
+            elif constant is staticmethod:
+                key = "(PyObject *)&PyStaticMethod_Type"
+            elif constant is classmethod:
+                key = "(PyObject *)&PyClassMethod_Type"
+            elif constant is bytearray:
+                key = "(PyObject *)&PyByteArray_Type"
+            elif constant is enumerate:
+                key = "(PyObject *)&PyEnum_Type"
+            elif constant is frozenset:
+                key = "(PyObject *)&PyFrozenSet_Type"
+            elif python_version >= 270 and constant is memoryview:
+                key = "(PyObject *)&PyMemoryView_Type"
+            elif python_version < 300 and constant is basestring: # pylint: disable=I0021,undefined-variable
+                key = "(PyObject *)&PyBaseString_Type"
+            elif python_version < 300 and constant is xrange: # pylint: disable=I0021,undefined-variable
+                key = "(PyObject *)&PyRange_Type"
+            elif constant in builtin_anon_values:
+                key = "(PyObject *)" + builtin_anon_codes[builtin_anon_values[constant]]
+            elif constant in builtin_exception_values_list:
+                key = "(PyObject *)PyExc_%s" % constant.__name__
+            else:
+                type_name = constant.__name__
+
+                if constant is int and python_version >= 300:
+                    type_name = "long"
+                elif constant is str:
+                    type_name = "string" if python_version < 300 else "unicode"
+
+                key = "(PyObject *)&Py%s_Type" % type_name.title()
         else:
             key = "const_" + namifyConstant(constant)
 
@@ -503,21 +826,154 @@ class PythonGlobalContext:
         return self.constants
 
 
-class FrameDeclarationsMixin:
+class FrameDeclarationsMixin(object):
     def __init__(self):
         self.frame_declarations = []
 
+        # Frame is active or not, default not.
+        self.frame_variables_stack = [""]
+        # Type descriptions of the current frame.
+        self.frame_type_descriptions = [()]
+
+        # Types of variables for current frame.
+        self.frame_variable_types = {}
+
+        self.frames_used = 0
+
+        # Currently active frame stack inside the context.
+        self.frame_stack = [None]
+
+        self.locals_dict_names = set()
+
+
+    def getFrameHandle(self):
+        return self.frame_stack[-1]
+
+    def pushFrameHandle(self, frame_handle):
+        self.frames_used += 1
+
+        if self.frames_used > 1:
+            frame_handle += "_%d" % self.frames_used
+
+        self.frame_stack.append(frame_handle)
+        return self.frame_stack[-1]
+
+    def popFrameHandle(self):
+        result = self.frame_stack[-1]
+        del self.frame_stack[-1]
+
+        return result
+
+    def getFramesCount(self):
+        return self.frames_used
+
     def addFrameDeclaration(self, frame_decl):
         self.frame_declarations.append(frame_decl)
 
     def getFrameDeclarations(self):
-        return self.frame_declarations
+        return self.frame_declarations + [
+            "NUITKA_MAY_BE_UNUSED char const *type_description_%d = NULL;" % (i+1)
+            for i in
+            range(self.getFramesCount())
+        ]
+
+    def pushFrameVariables(self, frame_variables):
+        """ Set current the frame variables. """
+        self.frame_variables_stack.append(frame_variables)
+        self.frame_type_descriptions.append(set())
+
+    def popFrameVariables(self):
+        """ End of frame, restore previous ones. """
+        del self.frame_variables_stack[-1]
+        del self.frame_type_descriptions[-1]
+
+    def setVariableType(self, variable, variable_code_name, variable_c_type):
+        assert variable.isLocalVariable(), variable
+
+        self.frame_variable_types[variable] = variable_code_name, variable_c_type.getTypeIndicator()
+
+    def getFrameVariableTypeDescriptions(self):
+        return self.frame_type_descriptions[-1]
+
+    def getFrameVariableTypeDescriptionName(self):
+        return "type_description_%d" % (len(self.frame_stack) - 1)
+
+    def getFrameVariableTypeDescription(self):
+        result = "".join(
+            self.frame_variable_types.get(variable, ("NULL", 'N'))[1]
+            for variable in
+            self.frame_variables_stack[-1]
+        )
+
+        if result:
+            self.frame_type_descriptions[-1].add(result)
+
+        return result
+
+    def getFrameVariableCodeNames(self):
+        result = []
+
+        for variable in self.frame_variables_stack[-1]:
+            variable_code_name, variable_code_type = self.frame_variable_types.get(variable, ("NULL", 'N'))
+
+            if variable_code_type in ('b',):
+                result.append("(int)" + variable_code_name)
+            else:
+                result.append(variable_code_name)
+
+        return result
+
+    def getLocalsDictNames(self):
+        return self.locals_dict_names
+
+    def addLocalsDictName(self, locals_dict_name):
+        self.locals_dict_names.add(locals_dict_name)
 
 
-class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
-                          FrameDeclarationsMixin):
+class ReturnReleaseModeMixin(object):
+    def __init__(self):
+        self.return_release_mode = False
+
+        self.return_exit = None
+
+    def setReturnReleaseMode(self, value):
+        result = self.return_release_mode
+        self.return_release_mode = value
+        return result
+
+    def getReturnReleaseMode(self):
+        return self.return_release_mode
+
+    def setReturnTarget(self, label):
+        result = self.return_exit
+        self.return_exit = label
+        return result
+
+    def getReturnTarget(self):
+        return self.return_exit
+
+
+class ReturnValueNameMixin(object):
+    def __init__(self):
+        self.return_name = None
+
+    def getReturnValueName(self):
+        if self.return_name is None:
+            self.return_name = self.allocateTempName("return_value", unique = True)
+
+        return self.return_name
+
+    def setReturnValueName(self, value):
+        result = self.return_name
+        self.return_name = value
+        return result
+
+
+class PythonModuleContext(TempMixin, CodeObjectsMixin,
+                          FrameDeclarationsMixin, ReturnReleaseModeMixin,
+                          ReturnValueNameMixin, PythonContextBase):
     # Plenty of attributes, because it's storing so many different things.
-    # pylint: disable=R0902
+    # pylint: disable=too-many-instance-attributes
 
     def __init__(self, module, module_name, code_name, filename, global_context):
         PythonContextBase.__init__(self)
@@ -525,6 +981,10 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
         TempMixin.__init__(self)
         CodeObjectsMixin.__init__(self)
         FrameDeclarationsMixin.__init__(self)
+        ReturnReleaseModeMixin.__init__(self)
+
+        # TODO: For outlines bodies.
+        ReturnValueNameMixin.__init__(self)
 
         self.module = module
         self.name = module_name
@@ -538,14 +998,8 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
 
         self.constants = set()
 
-        self.return_release_mode = False
-
         self.frame_handle = None
 
-        self.return_exit = True
-
-        self.return_name = None
-
         self.needs_module_filename_object = False
 
     def __repr__(self):
@@ -554,18 +1008,12 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
     def getOwner(self):
         return self.module
 
+    def getEntryPoint(self):
+        return self.module
+
     def isCompiledPythonModule(self):
         return True
 
-    def hasLocalsDict(self):
-        return False
-
-    def getFrameHandle(self):
-        return self.frame_handle
-
-    def setFrameHandle(self, frame_handle):
-        self.frame_handle = frame_handle
-
     def getName(self):
         return self.name
 
@@ -582,12 +1030,6 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
     def getModuleCodeName(self):
         return self.code_name
 
-    # There cannot be local variable in modules no need to consider the name.
-    # pylint: disable=W0613
-    def hasClosureVariable(self, var_name):
-        return False
-    # pylint: enable=W0613
-
     def setFrameGuardMode(self, guard_mode):
         assert guard_mode == "once"
 
@@ -610,30 +1052,6 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
     def getDeclarations(self):
         return self.declaration_codes
 
-    def getReturnValueName(self):
-        return self.return_name
-
-    def setReturnValueName(self, value):
-        result = self.return_name
-        self.return_name = value
-        return result
-
-    def setReturnReleaseMode(self, value):
-        result = self.return_release_mode
-        self.return_release_mode = value
-        return result
-
-    def getReturnReleaseMode(self):
-        return self.return_release_mode
-
-    def getReturnTarget(self):
-        return self.return_exit
-
-    def setReturnTarget(self, label):
-        result = self.return_exit
-        self.return_exit = label
-        return result
-
     def mayRecurse(self):
         return False
 
@@ -656,8 +1074,9 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
         return self.needs_module_filename_object
 
 
-class PythonFunctionContext(PythonChildContextBase, TempMixin,
-                            FrameDeclarationsMixin):
+class PythonFunctionContext(FrameDeclarationsMixin,
+                            TempMixin, ReturnReleaseModeMixin,
+                            ReturnValueNameMixin, PythonChildContextBase):
     def __init__(self, parent, function):
         PythonChildContextBase.__init__(
             self,
@@ -666,21 +1085,19 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin,
 
         TempMixin.__init__(self)
         FrameDeclarationsMixin.__init__(self)
+        ReturnReleaseModeMixin.__init__(self)
+        ReturnValueNameMixin.__init__(self)
 
         self.function = function
 
-        self.return_exit = None
-
         self.setExceptionEscape("function_exception_exit")
         self.setReturnTarget("function_return_exit")
 
-        self.return_release_mode = False
-        self.return_name = None
-
         self.frame_handle = None
 
     def __repr__(self):
-        return "<PythonFunctionContext for %s '%s'>" % (
+        return "<%s for %s '%s'>" % (
+            self.__class__.__name__,
             "function" if not self.function.isExpressionClassBody() else "class",
             self.function.getName()
         )
@@ -691,51 +1108,15 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin,
     def getOwner(self):
         return self.function
 
-    def hasLocalsDict(self):
-        return self.function.hasLocalsDict()
-
-    def hasClosureVariable(self, var_name):
-        return var_name in self.function.getClosureVariableNames()
-
-    def getFrameHandle(self):
-        return self.frame_handle
-
-    def setFrameHandle(self, frame_handle):
-        self.frame_handle = frame_handle
-
-    def getReturnValueName(self):
-        if self.return_name is None:
-            self.return_name = self.allocateTempName("return_value", unique = True)
-
-        return self.return_name
-
-    def setReturnValueName(self, value):
-        result = self.return_name
-        self.return_name = value
-        return result
-
-    def getReturnTarget(self):
-        return self.return_exit
-
-    def setReturnTarget(self, label):
-        result = self.return_exit
-        self.return_exit = label
-        return result
-
-    def setReturnReleaseMode(self, value):
-        result = self.return_release_mode
-        self.return_release_mode = value
-        return result
-
-    def getReturnReleaseMode(self):
-        return self.return_release_mode
+    def getEntryPoint(self):
+        return self.function
 
     def mayRecurse(self):
         # TODO: Determine this at compile time for enhanced optimizations.
         return True
 
-    def getCodeObjectHandle(self, **kw):
-        return self.parent.getCodeObjectHandle(**kw)
+    def getCodeObjectHandle(self, code_object):
+        return self.parent.getCodeObjectHandle(code_object)
 
 
 class PythonFunctionDirectContext(PythonFunctionContext):
@@ -748,6 +1129,7 @@ class PythonFunctionDirectContext(PythonFunctionContext):
     def isForCreatedFunction(self):
         return False
 
+
 class PythonGeneratorObjectContext(PythonFunctionContext):
     def isForDirectCall(self):
         return False
@@ -758,64 +1140,11 @@ class PythonGeneratorObjectContext(PythonFunctionContext):
     def isForCreatedFunction(self):
         return False
 
-
-class PythonCoroutineObjectContext(PythonGeneratorObjectContext):
-    pass
-
-
-class PythonFunctionCreatedContext(PythonFunctionContext):
-    def isForDirectCall(self):
-        return False
-
-    def isForCreatedFunction(self):
-        return True
-
-
-class PythonStatementCContext(PythonChildContextBase):
-    def __init__(self, parent):
-        PythonChildContextBase.__init__(
-            self,
-            parent = parent
-        )
-
-        self.cleanup_names = []
-
-        self.current_source_ref = None
-        self.last_source_ref = None
-
-    def getOwner(self):
-        return self.parent.getOwner()
-
-    def isCompiledPythonModule(self):
-        return self.parent.isCompiledPythonModule()
-
-    def getFunction(self):
-        return self.parent.getFunction()
-
-    def hasLocalsDict(self):
-        return self.parent.hasLocalsDict()
-
-    def isForDirectCall(self):
-        return self.parent.isForDirectCall()
-
-    def allocateTempName(self, base_name, type_code = "PyObject *",
-                         unique = False):
-        return self.parent.allocateTempName(base_name, type_code, unique)
-
-    def getIntResName(self):
-        return self.allocateTempName("res", "int", unique = True)
-
-    def getBoolResName(self):
-        return self.allocateTempName("result", "bool", unique = True)
-
-    def getReturnValueName(self):
-        return self.parent.getReturnValueName()
-
-    def setReturnValueName(self, value):
-        return self.parent.setReturnValueName(value)
+    def getContextObjectName(self):
+        return "generator"
 
     def getGeneratorReturnValueName(self):
-        if python_version >= 330:
+        if python_version >= 300:
             return self.allocateTempName(
                 "return_value",
                 "PyObject *",
@@ -828,6 +1157,75 @@ class PythonStatementCContext(PythonChildContextBase):
                 unique = True
             )
 
+
+class PythonCoroutineObjectContext(PythonGeneratorObjectContext):
+    def getContextObjectName(self):
+        return "coroutine"
+
+
+class PythonAsyncgenObjectContext(PythonGeneratorObjectContext):
+    def getContextObjectName(self):
+        return "asyncgen"
+
+
+class PythonFunctionCreatedContext(PythonFunctionContext):
+    def isForDirectCall(self):
+        return False
+
+    def isForCreatedFunction(self):
+        return True
+
+
+class PythonFunctionOutlineContext(ReturnReleaseModeMixin,
+                                   ReturnValueNameMixin, PythonChildContextBase):
+    def __init__(self, parent, outline):
+        PythonChildContextBase.__init__(
+            self,
+            parent = parent
+        )
+
+        ReturnReleaseModeMixin.__init__(self)
+        ReturnValueNameMixin.__init__(self)
+
+        self.outline = outline
+
+    def getOwner(self):
+        return self.outline
+
+    def getEntryPoint(self):
+        return self.outline.getEntryPoint()
+
+    def allocateLabel(self, label):
+        return self.parent.allocateLabel(label)
+
+    def allocateTempName(self, base_name, type_name = "PyObject *",
+                         unique = False):
+        return self.parent.allocateTempName(base_name, type_name, unique)
+
+    def hasTempName(self, base_name):
+        return self.parent.hasTempName(base_name)
+
+    def getCleanupTempnames(self):
+        return self.parent.getCleanupTempnames()
+
+    def addCleanupTempName(self, tmp_name):
+        self.parent.addCleanupTempName(tmp_name)
+
+    def removeCleanupTempName(self, tmp_name):
+        self.parent.removeCleanupTempName(tmp_name)
+
+    def needsCleanup(self, tmp_name):
+        return self.parent.needsCleanup(tmp_name)
+
+    def pushCleanupScope(self):
+        return self.parent.pushCleanupScope()
+
+    def popCleanupScope(self):
+        self.parent.popCleanupScope()
+
+    def getCodeObjectHandle(self, code_object):
+        return self.parent.getCodeObjectHandle(code_object)
+
     def getExceptionEscape(self):
         return self.parent.getExceptionEscape()
 
@@ -849,62 +1247,23 @@ class PythonStatementCContext(PythonChildContextBase):
     def getTrueBranchTarget(self):
         return self.parent.getTrueBranchTarget()
 
-    def setTrueBranchTarget(self, label):
-        return self.parent.setTrueBranchTarget(label)
-
     def getFalseBranchTarget(self):
         return self.parent.getFalseBranchTarget()
 
+    def setTrueBranchTarget(self, label):
+        self.parent.setTrueBranchTarget(label)
+
     def setFalseBranchTarget(self, label):
-        return self.parent.setFalseBranchTarget(label)
-
-    def getReturnTarget(self):
-        return self.parent.getReturnTarget()
-
-    def setReturnTarget(self, label):
-        return self.parent.setReturnTarget(label)
-
-    def getReturnReleaseMode(self):
-        return self.parent.getReturnReleaseMode()
-
-    def setReturnReleaseMode(self, value):
-        return self.parent.setReturnReleaseMode(value)
-
-    def allocateLabel(self, label):
-        return self.parent.allocateLabel(label)
-
-    def addCleanupTempName(self, tmp_name):
-        assert tmp_name not in self.cleanup_names, tmp_name
-
-        self.cleanup_names.append(tmp_name)
-
-    def removeCleanupTempName(self, tmp_name):
-        assert tmp_name in self.cleanup_names, tmp_name
-        self.cleanup_names.remove(tmp_name)
-
-    def needsCleanup(self, tmp_name):
-        return tmp_name in self.cleanup_names
-
-    def isUsed(self, tmp_name):
-        if tmp_name.startswith("tmp_unused_"):
-            return False
-        else:
-            return True
-
-    def forgetTempName(self, tmp_name):
-        self.parent.forgetTempName(tmp_name)
-
-    def getCleanupTempnames(self):
-        return self.cleanup_names
+        self.parent.setFalseBranchTarget(label)
 
     def getFrameHandle(self):
         return self.parent.getFrameHandle()
 
-    def setFrameHandle(self, frame_handle):
-        return self.parent.setFrameHandle(frame_handle)
+    def pushFrameHandle(self, frame_handle):
+        return self.parent.pushFrameHandle(frame_handle)
 
-    def allocateExceptionKeeperVariables(self):
-        return self.parent.allocateExceptionKeeperVariables()
+    def popFrameHandle(self):
+        return self.parent.popFrameHandle()
 
     def getExceptionKeeperVariables(self):
         return self.parent.getExceptionKeeperVariables()
@@ -912,8 +1271,14 @@ class PythonStatementCContext(PythonChildContextBase):
     def setExceptionKeeperVariables(self, keeper_vars):
         return self.parent.setExceptionKeeperVariables(keeper_vars)
 
-    def addExceptionPreserverVariables(self, count):
-        self.parent.addExceptionPreserverVariables(count)
+    def setVariableType(self, variable, variable_code_name, variable_c_type):
+        self.parent.setVariableType(variable, variable_code_name, variable_c_type)
+
+    def getIntResName(self):
+        return self.parent.getIntResName()
+
+    def getBoolResName(self):
+        return self.parent.getBoolResName()
 
     def needsExceptionVariables(self):
         return self.parent.needsExceptionVariables()
@@ -921,31 +1286,31 @@ class PythonStatementCContext(PythonChildContextBase):
     def markAsNeedsExceptionVariables(self):
         self.parent.markAsNeedsExceptionVariables()
 
+    def allocateExceptionKeeperVariables(self):
+        return self.parent.allocateExceptionKeeperVariables()
+
     def addFrameDeclaration(self, frame_decl):
         self.parent.addFrameDeclaration(frame_decl)
 
+    def isForDirectCall(self):
+        return self.parent.isForDirectCall()
+
     def mayRecurse(self):
-        return self.parent.mayRecurse()
+        # TODO: In modules, there could be loops with outlines, we could detect
+        # that.
+        return True
 
-    def getCodeObjectHandle(self, **kw):
-        return self.parent.getCodeObjectHandle(**kw)
+    def addLocalsDictName(self, locals_dict_name):
+        self.parent.addLocalsDictName(locals_dict_name)
 
-    def getCurrentSourceCodeReference(self):
-        return self.current_source_ref
+    def addExceptionPreserverVariables(self, count):
+        self.parent.addExceptionPreserverVariables(count)
 
-    def setCurrentSourceCodeReference(self, value):
-        result = self.current_source_ref
-        self.current_source_ref = value
+    def getTempNameInfos(self):
+        return self.parent.getTempNameInfos()
 
-        if value is not None:
-            self.last_source_ref = result
+    def forgetTempName(self, tmp_name):
+        self.parent.forgetTempName(tmp_name)
 
-        return result
-
-    def getLastSourceCodeReference(self):
-        result = self.last_source_ref
-        # self.last_source_ref = None
-        return result
-
-    def markAsNeedsModuleFilenameObject(self):
-        self.parent.markAsNeedsModuleFilenameObject()
+    def getContextObjectName(self):
+        return self.parent.getContextObjectName()
