@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -48,7 +48,7 @@ def generateBuiltinLocalsRefCode(to_name, expression, emit, context):
 def generateBuiltinLocalsCode(to_name, expression, emit, context):
     provider = expression.getParentVariableProvider()
 
-    variables = expression.getVariableVersions()
+    variable_traces = expression.getVariableTraces()
     updated   = expression.isExpressionBuiltinLocalsUpdated()
 
     # Locals is sorted of course.
@@ -92,47 +92,26 @@ Py_INCREF( %(to_name)s );""" % {
 
         initial = False
     else:
-        locals_scope = provider.getLocalsScope()
-
-        if locals_scope is not None:
-            # TODO: No need to use that variable normally.
-
-            locals_dict_name = locals_scope.getCodeName()
-
-            # TODO: Many uses could avoid adding a locals dict.
-            emit(
-                """\
-if (%(locals_dict)s == NULL) %(locals_dict)s = PyDict_New();
-%(to_name)s = PyDict_Copy( %(locals_dict)s );""" % {
-                    "to_name"     : to_name,
-                    "locals_dict" : locals_dict_name,
-                }
+        emit(
+            "%s = PyDict_New();" % (
+                to_name,
             )
-
-            context.addLocalsDictName(locals_dict_name)
-            initial = False
-        else:
-            emit(
-                "%s = PyDict_New();" % (
-                    to_name,
-                )
-            )
-
-            initial = True
+        )
 
         context.addCleanupTempName(to_name)
 
+        initial = True
         is_dict = True
 
-    for local_var, version in _sorted(variables):
+    for local_var, variable_trace in _sorted(variable_traces):
         _getVariableDictUpdateCode(
-            target_name = to_name,
-            variable    = local_var,
-            version     = version,
-            is_dict     = is_dict,
-            initial     = initial,
-            emit        = emit,
-            context     = context
+            target_name    = to_name,
+            variable       = local_var,
+            variable_trace = variable_trace,
+            is_dict        = is_dict,
+            initial        = initial,
+            emit           = emit,
+            context        = context
         )
 
 
@@ -175,12 +154,12 @@ def _getLocalVariableList(provider):
     ]
 
 
-def _getVariableDictUpdateCode(target_name, variable, version, initial, is_dict,
-                               emit, context):
+def _getVariableDictUpdateCode(target_name, variable, variable_trace, initial,
+                               is_dict, emit, context):
     # TODO: Variable could known to be set here, get a hand at that
     # information.
 
-    variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable, version)
+    variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable, variable_trace)
 
     test_code = variable_c_type.getLocalVariableInitTestCode(variable_code_name)
     access_code = variable_c_type.getLocalVariableObjectAccessCode(variable_code_name)
