@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -19,7 +19,8 @@
 #     limitations under the License.
 #
 
-import os, sys, subprocess
+import os
+import sys
 
 # The test runner needs "lxml" itself.
 try:
@@ -28,25 +29,28 @@ except ImportError:
     print("Warning, no 'lxml' module installed, cannot do XML based tests.")
     sys.exit(0)
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
-    my_print,
-    setup,
+
+from nuitka.tools.testing.Common import (  # isort:skip
+    check_output,
     convertUsing2to3,
     createSearchMode,
     hasModule,
-    check_output
+    my_print,
+    setup
 )
 
+
 python_version = setup()
 
 # The Python version used by Nuitka needs "lxml" too.
@@ -89,7 +93,9 @@ def isConstantExpression(expression):
 
     return kind.startswith("Constant") or \
            kind in ("ImportModuleHard",
-                    "ModuleFileAttributeRef")
+                    "ImportName",
+                    "ModuleAttributeFileRef",
+                    "ModuleLoaderRef")
 
 def checkSequence(statements):
     for statement in statements:
@@ -100,7 +106,7 @@ def checkSequence(statements):
 
             if not isConstantExpression(print_arg):
                 sys.exit(
-                    "%s: Error, print of non-constant %s." % (
+                    "%s: Error, print of non-constant '%s'." % (
                         getSourceRef(statement),
                         getKind(print_arg)
                     )
@@ -122,19 +128,26 @@ def checkSequence(statements):
                     if called_expression.attrib["builtin_name"] == "print":
                         continue
 
-        if kind == "Frame":
+        if kind == "FrameModule":
             checkSequence(getRole(statement, "statements"))
 
             continue
 
         if kind == "AssignmentVariable":
+            variable_name = statement.attrib["variable_name"]
+
+            # Ignore "__spec__" assignment for Python3.4, it is not going
+            # to be static.
+            if variable_name == "__spec__":
+                continue
+
             assign_source, = getRole(statement, "source")
 
             if getKind(assign_source) == "FunctionCreation":
                 continue
 
             elif not isConstantExpression(assign_source):
-                sys.exit("Error, assignment from non-constant %s." % getKind(assign_source))
+                sys.exit("Error, assignment from non-constant '%s'." % getKind(assign_source))
 
             continue
 
@@ -179,7 +192,7 @@ for filename in sorted(os.listdir('.')):
 
             # Coverage modules hates Nuitka to re-execute, and so we must avoid
             # that.
-            python_path = subprocess.check_output(
+            python_path = check_output(
                 [
                     os.environ["PYTHON"],
                     "-c"
@@ -209,25 +222,36 @@ for filename in sorted(os.listdir('.')):
         )
 
         # Parse the result into XML and check it.
-        root = lxml.etree.fromstring(result)
+        try:
+            root = lxml.etree.fromstring(result)
+        except lxml.etree.XMLSyntaxError:
+            print("Problematic XML output:")
+            print(result)
+            raise
+
+
         module_body = root[0]
         module_statements_sequence = module_body[0]
 
         assert len(module_statements_sequence) == 1
         module_statements = next(iter(module_statements_sequence))
 
-        checkSequence(module_statements)
+        try:
+            checkSequence(module_statements)
 
-        for function in root.xpath('role[@name="functions"]/node'):
-            function_body, = function.xpath('role[@name="body"]')
-            function_statements_sequence = function_body[0]
-            assert len(function_statements_sequence) == 1
-            function_statements = next(iter(function_statements_sequence))
+            for function in root.xpath('role[@name="functions"]/node'):
+                function_body, = function.xpath('role[@name="body"]')
+                function_statements_sequence = function_body[0]
+                assert len(function_statements_sequence) == 1
+                function_statements = next(iter(function_statements_sequence))
 
-            checkSequence(function_statements)
+                checkSequence(function_statements)
 
-        if changed:
-            os.unlink(filename)
+            if changed:
+                os.unlink(filename)
+        except SystemExit:
+            my_print("FAIL.")
+            raise
 
         my_print("OK.")
     else:
