@@ -50,6 +50,7 @@ tmp_dir = getTempDir()
 PACKAGE_LIST = (
     "nuitka",
     "nuitka/nodes",
+    "nuitka/specs",
     "nuitka/nodes/shapes",
     "nuitka/tree",
     "nuitka/importing",
@@ -223,7 +224,7 @@ def executePASS1():
             os.path.join("nuitka", "build", "inline_copy")
         )
 
-    shutil.copy(
+    shutil.copyfile(
         os.path.join(base_dir, "nuitka", "build", "SingleExe.scons"),
         os.path.join("nuitka", "build", "SingleExe.scons")
     )
